;; Load all the project code
(load "syntax.scm")
(load "meval.scm")
(define *meval-warn-define* #t)
(load "environment.scm")
(load "assert.scm") ; for testing

(define (m-eval exp env)
  (cond ((self-evaluating? exp) exp)
        ((variable? exp) (lookup-variable-value exp env))    
        ((quoted? exp) (text-of-quotation exp))
        ((assignment? exp) (eval-assignment exp env))
        ((definition? exp) (eval-definition exp env))
        ((if? exp) (eval-if exp env))
        ((lambda? exp)
         (make-procedure (lambda-parameters exp) (lambda-body exp) env))
        ((begin? exp) (eval-sequence (begin-actions exp) env))
        ((cond? exp) (m-eval (cond->if exp) env))
	((let? exp) (m-eval (let->application exp) env))
;	((until? exp) (eval-until exp env))
	((times-called? exp)                                   ;; NEW
	 (eval-times-called exp env))                          ;; NEW
        ((application? exp)
         (m-apply (m-eval (operator exp) env)
                (list-of-values (operands exp) env)))
        (else (error "Unknown expression type -- EVAL" exp))))

;; times-called special form
(define (times-called? exp)            ;; NEW
  (tagged-list? exp 'times-called))    ;; NEW
(define times-called-proc-name second) ;; NEW
(define (eval-times-called exp env)    ;; NEW
  (procedure-call-count
   (lookup-variable-value
    (times-called-proc-name exp)
    env)))

;; Procedure abstraction
(define (make-procedure parameters body env)
  (list 'procedure parameters body env 0))      ;; CHANGED
(define (compound-procedure? exp)
  (tagged-list? exp 'procedure))
(define (procedure-parameters p) (second p))
(define (procedure-body p) (third p))
(define (procedure-environment p) (fourth p))
(define (procedure-call-count p) (fifth p))     ;; NEW
(define (procedure-call-count-inc! p)           ;; NEW
  (set-car! (cddddr p) (inc (car (cddddr p))))) ;; NEW

;; Enhanced apply procedure: takes a primitive or compound
;; procedure and its argument list and evaluates the
;; procedure.  This version has been enhanced to increment
;; the call count of compound procedures
(define (m-apply procedure arguments)
  (cond ((primitive-procedure? procedure)
	 (apply-primitive-procedure procedure arguments))
	((compound-procedure? procedure)
	 (let ((return-value                     ;; NEW
		(eval-sequence
		 (procedure-body procedure)
		 (extend-environment (procedure-parameters procedure)
				     arguments
				     (procedure-environment procedure)))))
	   (procedure-call-count-inc! procedure) ;; NEW
	   return-value))
	(else (error "Unknown procedure type -- APPLY" procedure))))

; Reset the global environment to make sure it includes
; any tweaks we've made here.
(refresh-global-environment)

; Add the map procedure -- note that we can't just make map a primitive
; procedure since we need to m-eval the function f.  Its underlying
; representation is a tagged list, not a base-level Scheme procedure
; that can be evaluated directly.
(m-eval '(define map (lambda (f l)
		       (if (null? l)
			   #f
			   (cons (f (car l)) (map f (cdr l))))))
	the-global-environment)

; Evalute the test code.  As long a no error is generated, it all worked fine.
(m-eval '(define (dbl x) (+ x x)) the-global-environment)
(assert=      (m-eval '(times-called dbl) the-global-environment)            0)
(assert=      (m-eval '(dbl (dbl 4)) the-global-environment)                16)
(assert=      (m-eval '(times-called dbl) the-global-environment)            2)
(assert-equal (m-eval '(map dbl '(1 2 3 4)) the-global-environment) '(2 4 6 8))
(assert=      (m-eval '(times-called dbl) the-global-environment)            6)




